@@ -63,6 +63,7 @@ extern "C" {
 #define OP_RIGHTWARD_REPLACE 2
 #define OP_LEFTWARD_REPLACE 3
 #define OP_DOWNWARD_REPLACE 4
+#define OP_TWO_LEVEL_REPLACE 5
 
 /* Arrow types in fsmrules */
 #define ARROW_RIGHT 1
@@ -167,7 +168,7 @@ struct fsmcontexts {
 
 struct fsmrules {
     struct fsm *left;
-    struct fsm *right;   
+    struct fsm *right;
     struct fsm *right2;    /*Only needed for A -> B ... C rules*/
     struct fsm *cross_product;
     struct fsmrules *next;
@@ -285,6 +286,7 @@ FEXPORT struct fsm *fsm_add_loop(struct fsm *net, struct fsm *marker, int finals
 FEXPORT struct fsm *fsm_add_sink(struct fsm *net, int final);
 FEXPORT struct fsm *fsm_left_rewr(struct fsm *net, struct fsm *rewr);
 FEXPORT struct fsm *fsm_flatten(struct fsm *net, struct fsm *epsilon);
+FEXPORT struct fsm *fsm_unflatten(struct fsm *net, char *epsilon_sym, char *repeat_sym);
 FEXPORT struct fsm *fsm_close_sigma(struct fsm *net, int mode);
 FEXPORT char *fsm_network_to_char(struct fsm *net);
 
@@ -448,17 +450,6 @@ int sh_get_value(struct sh_handle *sh);
 /* Trie construction */
 /*********************/
 
-#ifndef ORIGINAL
-  #ifndef __cplusplus
-  #ifndef bool
-    #define bool int
-    #define false((bool)0)
-    #define true((bool)1)
-    #define _Bool bool
-  #endif
-  #endif
-#endif // #ifndef ORIGINAL
-
 struct trie_hash {
     char *insym;
     char *outsym;
@@ -468,7 +459,7 @@ struct trie_hash {
 };
 
 struct trie_states {
-    _Bool is_final;
+    Boolean is_final;
 };
 
 struct fsm_trie_handle {
@@ -504,7 +495,7 @@ struct fsm_read_handle {
     int sigma_list_size;
     struct fsm *net;
     unsigned char *lookuptable;
-    _Bool has_unknowns;
+    Boolean has_unknowns;
 };
 
 FEXPORT struct fsm_read_handle *fsm_read_init(struct fsm *net);
