@@ -178,7 +178,7 @@ template <class A>  class EncodeTable {
 
   static EncodeTable<A> *Read(istream &strm, const string &source);
 
-  const uint32 flags() const { return flags_ & kEncodeFlags; }
+  /*const*/ uint32 flags() const { return flags_ & kEncodeFlags; }
 
   int RefCount() const { return ref_count_.count(); }
   int IncrRefCount() { return ref_count_.Incr(); }
@@ -359,8 +359,8 @@ template <class A> class EncodeMapper {
     return props & mask;
   }
 
-  const uint32 flags() const { return flags_; }
-  const EncodeType type() const { return type_; }
+  /*const*/ uint32 flags() const { return flags_; }
+  /*const*/ EncodeType type() const { return type_; }
   const EncodeTable<A> &table() const { return *table_; }
 
   bool Write(ostream &strm, const string& source) {
