@@ -1,10 +1,10 @@
-// Copyright (c) 2016 University of Helsinki                          
-//                                                                    
-// This library is free software; you can redistribute it and/or      
-// modify it under the terms of the GNU Lesser General Public         
-// License as published by the Free Software Foundation; either       
+// Copyright (c) 2016 University of Helsinki
+//
+// This library is free software; you can redistribute it and/or
+// modify it under the terms of the GNU Lesser General Public
+// License as published by the Free Software Foundation; either
 // version 3 of the License, or (at your option) any later version.
-// See the file COPYING included with this distribution for more      
+// See the file COPYING included with this distribution for more
 // information.
 #ifndef _HFST_OL_TRANSDUCER_PMATCH_H_
 #define _HFST_OL_TRANSDUCER_PMATCH_H_
@@ -14,6 +14,7 @@
 #include <sstream>
 #include <algorithm>
 #include <ctime>
+#include "HfstTransducer.h"
 #include "transducer.h"
 
 namespace hfst_ol {
@@ -21,10 +22,8 @@ namespace hfst_ol {
     class PmatchTransducer;
     class PmatchContainer;
     struct Location;
-    class WeightedDoubleTape;
+    struct WeightedDoubleTape;
 
-    const unsigned int PMATCH_MAX_RECURSION_DEPTH = 5000;
-    
     typedef std::vector<PmatchTransducer *> RtnVector;
     typedef std::map<std::string, SymbolNumber> RtnNameMap;
     typedef std::vector<Location> LocationVector;
@@ -47,37 +46,6 @@ namespace hfst_ol {
                        Pmatch_input_mark,
                        SPECIALSYMBOL_NR_ITEMS};
 
-    struct SymbolPair
-    {
-        SymbolNumber input;
-        SymbolNumber output;
-    SymbolPair(void): input(0), output(0) {}
-    SymbolPair(SymbolNumber i, SymbolNumber o): input(i), output(o) {}
-    };
-
-    class DoubleTape: public std::vector<SymbolPair>
-    {
-    public:
-
-        void write(unsigned int pos, SymbolNumber in, SymbolNumber out)
-        {
-            while (pos >= this->size()) {
-                this->push_back(SymbolPair());
-            }
-            this->operator[](pos) = SymbolPair(in, out);
-        }
-
-        DoubleTape extract_slice(unsigned int start, unsigned int stop)
-        {
-            DoubleTape retval;
-            while(start < stop) {
-                retval.push_back(this->at(start));
-                ++start;
-            }
-            return retval;
-        }
-
-    };
 
     class PositionStack: public std::vector<unsigned int>
     {
@@ -89,18 +57,12 @@ namespace hfst_ol {
         unsigned int top(void) { return back(); }
     };
 
-    class WeightedDoubleTape: public DoubleTape
-    {
-    public:
-        Weight weight;
-    WeightedDoubleTape(DoubleTape dt, Weight w): DoubleTape(dt), weight(w) {}
-    };
-
     class PmatchAlphabet: public TransducerAlphabet {
     protected:
         RtnVector rtns;
         SymbolNumberVector special_symbols;
         std::map<SymbolNumber, std::string> end_tag_map;
+        std::map<SymbolNumber, std::string> words_like_map;
         RtnNameMap rtn_names;
 // For each symbol, either NO_SYMBOL for "no corresponding list" or an index into symbol_lists
         SymbolNumberVector symbol2lists;
@@ -112,24 +74,26 @@ namespace hfst_ol {
         SymbolNumberVector exclusionary_lists;
         std::vector<SymbolNumberVector> symbol_lists;
         std::vector<SymbolNumberVector> symbol_list_members;
-        std::vector<unsigned long int> counters;
+        std::vector<unsigned long> counters;
         SymbolNumberVector guards;
         std::vector<bool> printable_vector;
         bool is_end_tag(const SymbolNumber symbol) const;
         bool is_input_mark(const SymbolNumber symbol) const;
+        bool is_like_arc(const SymbolNumber symbol) const;
         bool is_guard(const SymbolNumber symbol) const;
         bool is_counter(const SymbolNumber symbol) const;
         std::string end_tag(const SymbolNumber symbol);
         std::string start_tag(const SymbolNumber symbol);
-        bool extract_tags;
+        PmatchContainer * container;
 
     public:
-        PmatchAlphabet(std::istream& is, SymbolNumber symbol_count);
-        PmatchAlphabet(TransducerAlphabet const & a);
+        PmatchAlphabet(std::istream& is, SymbolNumber symbol_count, PmatchContainer * cont);
+        PmatchAlphabet(TransducerAlphabet const & a, PmatchContainer * cont);
         PmatchAlphabet(void);
         ~PmatchAlphabet(void);
         virtual void add_symbol(const std::string & symbol);
         static bool is_end_tag(const std::string & symbol);
+        static bool is_like_arc(const std::string & symbol);
         static bool is_insertion(const std::string & symbol);
         static bool is_guard(const std::string & symbol);
         static bool is_list(const std::string & symbol);
@@ -174,7 +138,18 @@ namespace hfst_ol {
         LocationVectorVector locations;
         std::vector<char> possible_first_symbols;
         bool verbose;
+        
+        bool count_patterns;
+        bool delete_patterns;
+        bool extract_patterns;
         bool locate_mode;
+        bool mark_patterns;
+        size_t max_context_length;
+        size_t max_recursion;
+        bool need_separators;
+
+        unsigned long line_number;
+        std::map<std::string, size_t> pattern_counts;
         bool profile_mode;
         bool single_codepoint_tokenization;
         unsigned int recursion_depth_left;
@@ -187,15 +162,19 @@ namespace hfst_ol {
         // A flag to set for when time has been overstepped
         bool limit_reached;
 
+        void collect_first_symbols(void);
+
     public:
 
         PmatchContainer(std::istream & is);
         PmatchContainer(Transducer * toplevel);
+        PmatchContainer(std::vector<hfst::HfstTransducer> transducers);
         PmatchContainer(void);
         ~PmatchContainer(void);
 
-        unsigned long line_number;
 
+        void set_properties(void);
+        void set_properties(std::map<std::string, std::string> & properties);
         void initialize_input(const char * input);
         bool has_unsatisfied_rtns(void) const;
         std::string get_unsatisfied_rtn_name(void) const;
@@ -203,9 +182,10 @@ namespace hfst_ol {
         void process(const std::string & input);
         std::string match(const std::string & input,
                           double time_cutoff = 0.0);
-        LocationVectorVector locate(std::string & input,
+        LocationVectorVector locate(const std::string & input,
                                     double time_cutoff = 0.0);
         std::string get_profiling_info(void);
+        std::string get_pattern_count_info(void);
         bool has_queued_input(unsigned int input_pos);
         bool not_possible_first_symbol(SymbolNumber sym)
         {
@@ -219,12 +199,24 @@ namespace hfst_ol {
         void copy_to_output(SymbolNumber input, SymbolNumber output);
         std::string stringify_output(void);
 //        LocationVector locatefy_output(void);
-        static std::string parse_name_from_hfst3_header(std::istream & f);
+        static std::map<std::string, std::string> parse_hfst3_header(std::istream & f);
         void set_verbose(bool b) { verbose = b; }
-        void set_extract_tags_mode(bool b)
-            { alphabet.extract_tags = b; }
+        void set_locate_mode(bool b) { locate_mode = b; }
+        void set_extract_patterns(bool b)
+            { extract_patterns = b; }
         void set_single_codepoint_tokenization(bool b)
             { single_codepoint_tokenization = b; }
+        void set_count_patterns(bool b)
+            { count_patterns = b; }
+        void set_delete_patterns(bool b)
+            { delete_patterns = b; }
+        void set_mark_patterns(bool b)
+            { mark_patterns = b; }
+        void set_max_recursion(size_t max)
+            { max_recursion = max; }
+        void set_max_context(size_t max)
+            { max_context_length = max; }
+        bool is_in_locate_mode(void) { return locate_mode; }
         void set_profile(bool b) { profile_mode = b; }
         bool try_recurse(void)
         {
@@ -236,9 +228,10 @@ namespace hfst_ol {
             }
         }
         void unrecurse(void) { ++recursion_depth_left; }
-        void reset_recursion(void) { recursion_depth_left = PMATCH_MAX_RECURSION_DEPTH; }
+        void reset_recursion(void) { recursion_depth_left = (unsigned int)max_recursion; }
 
         friend class PmatchTransducer;
+        friend class PmatchAlphabet;
     };
 
     struct Location
@@ -249,8 +242,10 @@ namespace hfst_ol {
         std::string output;
         std::string tag;
         Weight weight;
-        std::vector<size_t> input_parts;
-        std::vector<size_t> output_parts;
+        std::vector<size_t> input_parts; // indices in input_symbol_strings
+        std::vector<size_t> output_parts; // indices in output_symbol_strings
+        std::vector<std::string> input_symbol_strings;
+        std::vector<std::string> output_symbol_strings;
 
         bool operator<(Location rhs) const
             { return this->weight < rhs.weight; }
@@ -275,6 +270,7 @@ namespace hfst_ol {
         {
             hfst::FdState<SymbolNumber> flag_state;
             char tape_step;
+            size_t max_context_length_remaining;
             unsigned int context_placeholder;
             ContextChecking context;
             bool default_symbol_trap;
@@ -336,6 +332,8 @@ namespace hfst_ol {
             }
         }
 
+        void match_like_arc(unsigned int input_pos, unsigned int tape_pos);
+
         // The mutually recursive lookup-handling functions
 
         void take_epsilons(unsigned int input_pos,
