@@ -1,10 +1,10 @@
-// Copyright (c) 2016 University of Helsinki                          
-//                                                                    
-// This library is free software; you can redistribute it and/or      
-// modify it under the terms of the GNU Lesser General Public         
-// License as published by the Free Software Foundation; either       
+// Copyright (c) 2016 University of Helsinki
+//
+// This library is free software; you can redistribute it and/or
+// modify it under the terms of the GNU Lesser General Public
+// License as published by the Free Software Foundation; either
 // version 3 of the License, or (at your option) any later version.
-// See the file COPYING included with this distribution for more      
+// See the file COPYING included with this distribution for more
 // information.
 
 //! @file PmatchCompiler.cc
@@ -45,7 +45,12 @@ std::map<std::string, HfstTransducer*>
 PmatchCompiler::compile(const std::string& pmatch)
 {
     return hfst::pmatch::compile(pmatch, definitions_, format_,
-                                 verbose, flatten);
+                                 verbose, flatten, includedir);
+}
+
+void PmatchCompiler::set_include_path(std::string path)
+{
+    includedir = path;
 }
 
 }}
@@ -165,7 +170,7 @@ main(int, char**)
 //     std::cout << "(a a | b c)...";
 //     HfstTransducer* ofstAaOrBc = ofstPmatch.compile("a a | b c");
 //     assert(ofstAaOrBc != 0);
-//     assert(ofstAaOrBc->compare(HfstTransducer(basicAaOrBc, 
+//     assert(ofstAaOrBc->compare(HfstTransducer(basicAaOrBc,
 //                                               TROPICAL_OPENFST_TYPE)));
 //     delete ofstAaOrBc;
 // #endif
