@@ -1,10 +1,10 @@
-// Copyright (c) 2016 University of Helsinki                          
-//                                                                    
-// This library is free software; you can redistribute it and/or      
-// modify it under the terms of the GNU Lesser General Public         
-// License as published by the Free Software Foundation; either       
+// Copyright (c) 2016 University of Helsinki
+//
+// This library is free software; you can redistribute it and/or
+// modify it under the terms of the GNU Lesser General Public
+// License as published by the Free Software Foundation; either
 // version 3 of the License, or (at your option) any later version.
-// See the file COPYING included with this distribution for more      
+// See the file COPYING included with this distribution for more
 // information.
 
 //! @file PmatchCompiler.h
@@ -30,8 +30,8 @@
 #include <string>
 #include "../HfstDataTypes.h"
 
-namespace hfst { 
-//! @brief hfst::pmatch namespace is used for all functions related to Xerox 
+namespace hfst {
+//! @brief hfst::pmatch namespace is used for all functions related to Xerox
 //! Regular Expresisions (PMATCH) parsing.
 namespace pmatch {
 //! @brief A compiler holding information needed to compile PMATCHs.
@@ -40,6 +40,7 @@ class PmatchCompiler
 private:
     bool flatten;
     bool verbose;
+    std::string includedir;
   public:
   //! @brief Construct compiler for unknown format transducers.
   PmatchCompiler();
@@ -58,6 +59,7 @@ private:
   //!        May return a pointer to @e empty transducer on non-fatal error.
   //!        A null pointer is returned on fatal error, if abort is not called.
   std::map<std::string, HfstTransducer*> compile(const std::string& pmatch);
+  void set_include_path(std::string path);
 
   private:
   std::map<std::string,hfst::HfstTransducer*> definitions_;
