@@ -10,8 +10,17 @@ from distutils.core import setup, Extension
 libhfst_src_path = '../libhfst/src/'
 absolute_libhfst_src_path = os.path.abspath(libhfst_src_path)
 
-# when making the debian package, replace extra_link_args
+extra_link_arguments = []
+# If you wish to link to the local HFST library, replace the above with:
+# extra_link_arguments = ["-Wl,-rpath=" + absolute_libhfst_src_path + "/.libs"]
+
+# When making the debian package, replace extra_link_args
 # with ["-L/usr/lib/", "-Wl,-rpath=/usr/lib/"]
+
+# If you wish to link hfst c++ library statically, use:
+# library_dirs = []
+# libraries = []
+# extra_objects = absolute_libhfst_src_path + "/.libs/libhfst.a"
 libhfst_module = Extension('_libhfst',
                            language = "c++",
                            sources = ["libhfst.i"],
@@ -20,18 +29,20 @@ libhfst_module = Extension('_libhfst',
                            include_dirs = [absolute_libhfst_src_path],
                            library_dirs = [absolute_libhfst_src_path + "/.libs"],
                            libraries = ["hfst"],
-                           extra_link_args = []
+                           extra_compile_args = ["-std=c++0x"],
+                           extra_link_args = extra_link_arguments
                            )
-# when making the windows package, replace data_files with
+# When making the windows package, replace data_files with
 # ["libhfst-NN.dll", "libgcc_s_seh-1.dll"] or
 # ["libhfst-NN.dll", "libgcc_s_dw2-1.dll"] or
 setup(name = 'libhfst_swig',
-      version = '3.10.0_beta',
+      version = '3.12.2_beta',
       author = 'HFST team',
       author_email = 'hfst-bugs@helsinki.fi',
-      url = 'http://hfst.sourceforge.net',
-      description = 'SWIG-bound new libhfst interface',
+      url = 'http://hfst.github.io/',
+      description = 'SWIG-bound hfst interface',
       ext_modules = [libhfst_module],
       py_modules = ["libhfst"],
+      packages = ["hfst", "hfst.exceptions", "hfst.sfst_rules", "hfst.xerox_rules"],
       data_files = []
       )
