@@ -94,7 +94,7 @@ parse_options(int argc, char** argv)
           {0,0,0,0}
         };
         int option_index = 0;
-        char c = getopt_long(argc, argv, HFST_GETOPT_COMMON_SHORT
+        int c = getopt_long(argc, argv, HFST_GETOPT_COMMON_SHORT
                              HFST_GETOPT_BINARY_SHORT "H",
                              long_options, &option_index);
         if (-1 == c)
@@ -175,7 +175,7 @@ disjunct_streams(HfstInputStream& firststream, HfstInputStream& secondstream)
         char* secondname = hfst_get_name(*second, secondfilename);
         if (transducer_n_first == 1)
         {
-            verbose_printf("Disjuncting %s and %s...\n", firstname, 
+            verbose_printf("Disjuncting %s and %s...\n", firstname,
                         secondname);
         }
         else
@@ -207,13 +207,13 @@ disjunct_streams(HfstInputStream& firststream, HfstInputStream& secondstream)
         hfst_set_formula(*first, *first, *second, "∪");
         outstream << *first;
 
-        continueReading = firststream.is_good() && 
+        continueReading = firststream.is_good() &&
           (secondstream.is_good() || transducer_n_second == 1);
 
         delete first;
         first=0;
         // delete the transducer of second stream, unless we continue reading
-        // the first stream and there is only one transducer in the second 
+        // the first stream and there is only one transducer in the second
         // stream
         if ((continueReading && secondstream.is_good()) || not continueReading)
           {
@@ -228,10 +228,10 @@ disjunct_streams(HfstInputStream& firststream, HfstInputStream& secondstream)
     
     if (firststream.is_good())
       {
-        error(EXIT_FAILURE, 0, 
+        error(EXIT_FAILURE, 0,
               "second input '%s' contains fewer transducers than first input"
               " '%s'; this is only possible if the second input contains"
-              " exactly one transducer", 
+              " exactly one transducer",
               secondfilename, firstfilename);
       }
 
@@ -273,7 +273,7 @@ int main( int argc, char **argv ) {
     {
         fclose(outfile);
     }
-    verbose_printf("Reading from %s and %s, writing to %s\n", 
+    verbose_printf("Reading from %s and %s, writing to %s\n",
         firstfilename, secondfilename, outfilename);
     // here starts the buffer handling part
     HfstInputStream* firststream = NULL;
