@@ -61,8 +61,8 @@ static bool print_weight = false;
 void
 print_usage()
 {
-  // c.f. http://www.gnu.org/prep/standards/standards.html#g_t_002d_002dhelp                                                                                                                                                               
-  // Usage line                                                                                                                                                                                                                            
+  // c.f. http://www.gnu.org/prep/standards/standards.html#g_t_002d_002dhelp
+  // Usage line
   fprintf(message_out, "Usage: %s [OPTIONS...]\n"
           "Compile XFST scripts or execute XFST commands interactively\n"
           "\n", "hfst-xfst" /*program_name*/);
@@ -70,8 +70,8 @@ print_usage()
   print_common_program_options(message_out);
   fprintf(message_out, "\n");
   fprintf(message_out, "Xfst-specific options:\n");
-  fprintf(message_out, 
-          "  -e, --execute=CMD        Execute command CMD on startup\n" 
+  fprintf(message_out,
+          "  -e, --execute=CMD        Execute command CMD on startup\n"
           "  -f, --format=FMT         Write result using FMT as backend format\n"
           "  -F, --scriptfile=FILE    Read commands from FILE, and quit\n"
           "  -l, --startupfile=FILE   Read commands from FILE on startup\n"
@@ -107,7 +107,7 @@ int
 parse_options(int argc, char** argv)
 {
   extend_options_getenv(&argc, &argv);
-  // use of this function requires options are settable on global scope                                                                                                                                                                    
+  // use of this function requires options are settable on global scope
   while (true)
     {
         static const struct option long_options[] =
@@ -126,7 +126,7 @@ parse_options(int argc, char** argv)
           };
         int option_index = 0;
         // add tool-specific options here
-        char c = getopt_long(argc, argv, HFST_GETOPT_COMMON_SHORT "f:F:e:l:p::rwk",
+        int c = getopt_long(argc, argv, HFST_GETOPT_COMMON_SHORT "f:F:e:l:p::rwk",
                              long_options, &option_index);
         if (-1 == c)
           {
@@ -218,7 +218,7 @@ parse_options(int argc, char** argv)
 int parse_file(const char* filename, hfst::xfst::XfstCompiler &comp)
 {
   char* line = hfst_file_to_mem(filename);
-  if (NULL == line) 
+  if (NULL == line)
     {
       error(EXIT_FAILURE, 0, "error when reading file %s\n", filename);
       return EXIT_FAILURE;
@@ -237,7 +237,7 @@ void insert_zeroes(char * array, unsigned int number)
 {
   for (unsigned int i=0; i<number; i++)
     {
-      array[i] = '\0'; 
+      array[i] = '\0';
     }
 }
 
@@ -275,7 +275,7 @@ static int smatch;
 // By Mans Hulden
 char *my_generator(const char *text, int state) {
   static int list_index, list_index2, len, nummatches;
-  char *name;
+  const char *name;
   text = rl_line_buffer;
   if (!state) {
     list_index = 0;
@@ -288,7 +288,7 @@ char *my_generator(const char *text, int state) {
     list_index++;
 
     if (strncmp (name, text, len) == 0) {
-      nummatches++;      
+      nummatches++;
       return(strdup(name+smatch));
     }
   }
@@ -407,7 +407,7 @@ int main(int argc, char** argv)
         }
     }
 
-  if (pipe_input) 
+  if (pipe_input)
     {
       verbose_printf("Reading from standard input...\n");
       comp.setReadInteractiveTextFromStdin(false);
@@ -458,7 +458,7 @@ int main(int argc, char** argv)
 #else
       char line [MAX_LINE_LENGTH];
       insert_zeroes(line, MAX_LINE_LENGTH);
-      while (cin.getline(line, MAX_LINE_LENGTH))
+      while (std::cin.getline(line, MAX_LINE_LENGTH))
         {
           std::string linestr(line);
           expression += linestr;
@@ -525,6 +525,8 @@ int main(int argc, char** argv)
           if (buf[0] != 0) {
             add_history(expression.c_str()); }
 
+          free(buf);
+
           if (0 != comp.parse_line((expression + "\n").c_str()))
             {
 #ifdef WINDOWS
@@ -547,14 +549,13 @@ int main(int argc, char** argv)
           free(promptline);
           promptline = (!silent) ? comp.get_prompt() : strdup("");
         }
-      free(buf);
+
       free(promptline);
 #else
       fprintf(stderr, "ERROR: missing readline library\n");
       return EXIT_FAILURE;
 #endif
     }
-
   return EXIT_SUCCESS;
 }
 
